function model = scenemodel_annotate(model)
  close all;
  opt = get_opt();
  
  for i=1:4
    figure(i); set(gcf, 'WindowStyle', 'docked');
  end
  opt.mask = false(opt.H, opt.W);
  opt = disp_frontal(model, opt);
  while true
    opt.mask = false(opt.H, opt.W);
    opt.id = [];
    selected = false(numel(opt.label_names), 1);
    clc;
    fprintf('Click on one region, then press "enter" or "e".\n');
    fprintf('Press "g" to annotate ground or press "q" to quit\n');

    model = scenemodel_translate(model);
    figure(opt.fig_preview);
    clf;
    scenemodel_render(model);
    set(gca, 'Position', [0 0 1 1]);

    while true
      opt = disp_frontal(model, opt);
      if ~isempty(opt.id)
        fprintf('%s selected.\n', opt.label_names{opt.id(1)});
      else
        fprintf('Nothing selected.\n');
      end
      % prioritize on getting the floor
      if numel(model.objects)==0
        b='g';
      else
        [x y b] = ginput(1);
      end
      
      if b=='q'
        return;
      elseif isempty(b) && ~isempty(opt.id) % if enter is pressed
        %annotate_layout();
        %
        if ismember(opt.label_names{opt.id(1)}, opt.layout_class)
          opt.opening = false;
          [model, opt] = annotate_layout(model, opt);
        else
          [model, opt] = annotate_extruded(model, opt);
        end
        break;
      elseif strcmp(char(b), 'e') && ~isempty(opt.id) % if "e" is pressed
        if ismember(opt.label_names{opt.id(1)}, opt.furniture_class)
          [model, opt] = annotate_furniture(model, opt);
        elseif ismember(opt.label_names{opt.id(1)}, opt.layout_class)
          opt.opening = true;
          [model, opt] = annotate_layout(model, opt);
        end
        break;
      elseif strcmp(char(b), char(1)) % a regular click
        x=round(x);y=round(y);
        if x>0 && y>0 && x<=opt.W && y<=opt.H
          if opt.gt(round(y), round(x))~=0
            if strcmp('floor', opt.label_names{ opt.gt(round(y), round(x))})
              continue;
            end
            selected(opt.gt(round(y), round(x))) = ~selected(opt.gt(round(y), round(x)));
            opt.id=find(selected);
            opt.mask(opt.gt==opt.gt(round(y), round(x))) = ~opt.mask(opt.gt==opt.gt(round(y), round(x)));
          end
        end
        continue;
      elseif strcmp(char(b), 'd')
        for i=1:numel(model.objects)
          if model.objects{i}.uid == opt.id(1)
            new_set = setdiff(1:numel(model.objects), i);
            model.objects = {model.objects{new_set}};
            break;
          end
        end
        fprintf('%s deleted.\n',  opt.label_names{opt.id(1)});
        break;
      elseif strcmp(char(b), 'g')
        opt.plane = [0 1 0 0];
        [model, opt] = annotate_floor(model, opt);
        break;
      end
    end
    
  end % main loop
  
end

function [model, opt] = annotate_furniture(model, opt)
  [model, opt] = annotate_extruded(model, opt);
  fnls = dir(['model/' opt.label_names{opt.id(1)} '*.mat']);
  fnls = {fnls.name};
  n = numel(fnls);
  
  R = eye(3); M = 1;
  for i=1:numel(model.objects)
    if model.objects{i}.uid == opt.id(1)
      old_id = i;
      if isfield(model.objects{i}.model, 'R')
        R = model.objects{i}.model.R;
      end
      if isfield(model.objects{i}.model, 'basemodel')
        [~, M] = ismember(model.objects{i}.model.basemodel, fnls);
      end
    end
  end
  if isempty(old_id)
    old_id = numel(model.objects)+1;
  end
  
  model.objects{old_id}.model.R = R;
  model.objects{old_id}.model.basemodel = fnls{M};
  model.objects{old_id}.model.type = 'furniture';
  
  
  while true
    clc; fprintf('Press "r" to correct orientation and "c" to change model.')
    
    figure(opt.fig_preview);
    clf;
    model = scenemodel_translate(model, opt);
    scenemodel_render(model, opt);
    [x y bt] = ginput(1);
    if isempty(bt)
      return;
    elseif strcmp(char(bt), 'r')
      R = R*[cos(pi/4) 0 sin(pi/4); 0 1 0 ; -sin(pi/4) 0 cos(pi/4)];
      model.objects{old_id}.model.R = R;
    elseif strcmp(char(bt), 'c')
      M = mod(M, n) + 1;
      model.objects{old_id}.model.basemodel = fnls{M};
    end
  end
end

function [model, opt] = annotate_extruded(model, opt)
  overhead2Dpts = []; horizontal2Dpts = []; old_id = [];
  for i=1:numel(model.objects)
    if model.objects{i}.uid == opt.id(1)
      if isfield(model.objects{i}.model, 'overhead2D')
        overhead2D = model.objects{i}.model.overhead2D.pts;
        s_pts = [overhead2D{:}];
        overhead2Dpts = [s_pts.x; s_pts.y; s_pts.z];
      end
      if isfield(model.objects{i}.model, 'horizontal2D')
        horizontal2D = model.objects{i}.model.horizontal2D.pts;
        s_pts = [horizontal2D{:}];
        horizontal2Dpts = [s_pts.x; s_pts.y; s_pts.z];
      end
      old_id = i;
    end
  end
  opt.plane = [0 1 0 0];
  [proj_y, proj_x]=helper_surfaceproj(opt.plane);
  if isempty(overhead2Dpts) || isempty(horizontal2Dpts)
    % default to bbox
    p3d = reshape(model.data.coord, [opt.W*opt.H 3]);
    p3d = p3d(opt.mask, :)*[proj_y; proj_x; opt.plane(1:3)]';
    [p1y, p2y]=pminmax(p3d(:,1));
    [p1x, p2x]=pminmax(p3d(:,2));
    [p1z, p2z]=pminmax(p3d(:,3));
    p1 = p1x*proj_x + p1y*proj_y + p1z*opt.plane(1:3);
    p2 = p2x*proj_x + p2y*proj_y + p2z*opt.plane(1:3);
    overhead2Dpts =    [p1(1) p1(2) p1(3); p1(1) p1(2) p2(3); p2(1) p1(2) p2(3); p2(1) p1(2) p1(3)]';
    horizontal2Dpts =  [p1(1) p1(2) p1(3); p1(1) p2(2) p1(3); p2(1) p2(2) p1(3); p2(1) p1(2) p1(3)]';
    for j=1:numel(model.data.info)
      if strcmp(model.data.info{j}.key, 'support')
        sp = model.data.info{j}.value(model.data.info{j}.value(:,1)==opt.id(1),2);
        
        if ~isempty(sp)
          sp = sp(1);
          if sp(1)<0
            horizontal2Dpts(2,[1,4])=-model.objects{1}.model.surfaces{1}.plane(4)/model.objects{1}.model.surfaces{1}.plane(2);
          end
          if sp(1)>=1
            if strcmp( opt.label_names{sp(1)}, 'floor')
              horizontal2Dpts(2,[1,4])=-model.objects{1}.model.surfaces{1}.plane(4)/model.objects{1}.model.surfaces{1}.plane(2);
            end
          end
        end
      end
    end
  end
  
  overhead2D = helper_annotate(overhead2Dpts, model, opt);
  opt.plane = [0 0 -1 0];
  horizontal2D = helper_annotate(horizontal2Dpts, model, opt);
  img_reshape = reshape(model.data.image, [opt.W*opt.H 3]);
  colors = double(median(img_reshape(opt.mask, :), 1));
  
  % update object
  if isempty(old_id)
    o = struct;
  else
    o = model.objects{old_id};
  end
  o.uid = opt.id(1);
  o.type = 'ExtrudedObject';
  colors = colors.^.3;
  o.model.app.color.x = colors(1);
  o.model.app.color.y = colors(2);
  o.model.app.color.z = colors(3);
  o.model.type = 'object';
  o.model.label = opt.label_names{opt.id(1)};
  o.model.projection2D = [0 1 0 0];
  o.model.overhead2D = overhead2D;
  o.model.horizontal2D = horizontal2D;
  if ~isempty(old_id)
    model.objects{old_id} = o;
  else
    model.objects{end+1} = o;
  end
  
end

function [model, opt] = annotate_layout(model, opt)
  overhead2Dpts = [];
  old_id = [];
  for i=1:numel(model.objects)
    if model.objects{i}.uid == opt.id(1)
      if isfield(model.objects{i}.model, 'overhead2D')
        overhead2D = model.objects{i}.model.overhead2D.pts;
        s_pts = [overhead2D{:}];
        overhead2Dpts = [s_pts.x; s_pts.y; s_pts.z];
      end
      old_id = i;
    end
  end
  
  if strcmp(opt.label_names{opt.id(1)}, 'ceiling')
    projection2D = [0 0 -1 0];
    opt.plane = [0 0 -1 0];
%     figure(opt.fig_horizontal);
    overhead2D = helper_annotate(overhead2Dpts, model, opt);
    
    opt.plane = [0 1 0 0];
%     figure(opt.fig_overhead);
    surfaces = detail_layoutsurfaces(overhead2D, projection2D, model, opt);
    
  elseif strcmp(opt.label_names{opt.id(1)}, 'wall')
    projection2D = [0 1 0 0];
    opt.plane = [0 1 0 0];
%     figure(opt.fig_overhead);
%     opt = disp_orth(model, opt);
    overhead2D = helper_annotate(overhead2Dpts, model, opt);
    
    figure(opt.fig_horizontal);
    opt.plane = [0 0 -1 0];
%     opt = disp_orth(model, opt);
%     figure(opt.fig_overhead);
    surfaces = detail_layoutsurfaces(overhead2D, projection2D, model, opt);
    
  end
  
  img_reshape = reshape(model.data.image, [opt.W*opt.H 3]);
  colors = double(median(img_reshape(opt.mask, :), 1));
  o = struct;
  o.uid = opt.id(1);
  o.type = 'ExtrudedObject';
  colors = colors.^.3;
  o.model.app.color.x = colors(1);
  o.model.app.color.y = colors(2);
  o.model.app.color.z = colors(3);
  o.model.type = 'layout';
  o.model.label = opt.label_names{opt.id(1)};
  o.model.projection2D = projection2D;
  o.model.overhead2D = overhead2D;
  o.model.surfaces = surfaces;
  if ~isempty(old_id)
    model.objects{old_id} = o;
  else
    model.objects{end+1} = o;
  end
end

function surface = detail_layoutsurfaces(overhead2D, projection2D, model, opt)
  p3d = reshape(model.data.coord, [opt.W*opt.H 3]);
  p3d = p3d*projection2D(1:3)';
  % default will be from the ceiling to the floor
  extrusion(1) = min(p3d(opt.mask));
  extrusion(2) = max(p3d(opt.mask));
  if opt.plane(3)==-1
    spts = [model.objects{1}.model.surfaces{1}.polygon.pts{:}];
    extrusion(1) = min([spts.y]);
    t = -inf;
    for j=1:numel(model.objects)
      if strcmp(model.objects{j}.model.label, 'ceiling')
        spts = [model.objects{j}.model.surfaces{1}.polygon.pts{:}];
        t = max(t, max([spts.y]));
      end
    end
    if ~isinf(t), extrusion(2)=t; end
  end
  surface = cell(numel(overhead2D.pts)-1, 1);
  for i=2:numel(overhead2D.pts)
    o_pts = [overhead2D.pts{i-1}.x overhead2D.pts{i-1}.y overhead2D.pts{i-1}.z; ...
             overhead2D.pts{i}.x overhead2D.pts{i}.y overhead2D.pts{i}.z];
    top_pts = repmat(projection2D(1:3)*min(extrusion), [size(o_pts, 1) 1]) + o_pts;
    bottom_pts = repmat(projection2D(1:3)*max(extrusion), [size(o_pts, 1) 1]) + o_pts;
    
    horizontal2Dpts = [top_pts; bottom_pts([2,1],:)]';
    [u d v]=svd([horizontal2Dpts' ones(size(horizontal2Dpts,2),1)]);
    surface{i-1}.plane = v(:,4)';
    surface{i-1}.plane = surface{i-1}.plane / sqrt( sum(surface{i-1}.plane(1:3).^2) );
    
    % annotate polygon
    opt.plane = surface{i-1}.plane;
    horizontal2D = helper_annotate(horizontal2Dpts, model, opt);
    s_pts = [horizontal2D.pts{:}];
    h_pts = [s_pts.x; s_pts.y; s_pts.z];
    h_pts_p = h_pts'*projection2D(1:3)';
    extrusion(1) = min(h_pts_p);
    extrusion(2) = max(h_pts_p);
    top_pts = repmat(projection2D(1:3)*min(extrusion), [size(o_pts, 1) 1]) + o_pts;
    bottom_pts = repmat(projection2D(1:3)*max(extrusion), [size(o_pts, 1) 1]) + o_pts;
    horizontal2Dpts = [top_pts; bottom_pts([2,1],:)];
    horizontal2D = struct;
    for j=1:size(horizontal2Dpts, 1)
      horizontal2D.pts{j}.x = horizontal2Dpts(j,1);
      horizontal2D.pts{j}.y = horizontal2Dpts(j,2);
      horizontal2D.pts{j}.z = horizontal2Dpts(j,3);
    end
    surface{i-1}.polygon = horizontal2D;
    [u d v]=svd([horizontal2Dpts ones([size(horizontal2Dpts,1), 1])]);
    surface{i-1}.plane = v(:,4);
    surface{i-1}.plane = surface{i-1}.plane / sqrt( sum(surface{i-1}.plane(1:3).^2) );
    
    if opt.opening
      n_opening = 0;
      while true
        clc; fprintf('Press "e" to add an openning or "enter" to skip\n');
        [x y bt] = ginput(1);
        if isempty(bt), break; end
        if bt=='e'
          poly = helper_annotate([], model, opt);
          n_opening = n_opening+1;
          surface{i-1}.opening{n_opening} = poly;
        end
      end
    end
  end
end

function [polygon, current_snap] = helper_annotate(p, model, opt)
  if abs(opt.plane(2))>abs(opt.plane(3))
    figure(opt.fig_overhead);
  else
    figure(opt.fig_horizontal);
  end
  opt = disp_orth(model, opt);
  snap = helper_snapping(model, opt);
  old_p = [];
  if ~isempty(p)
    old_p = ([opt.trans.proj_x*p; opt.trans.proj_y*p]*opt.trans.S+repmat(opt.trans.T, [1 size(p, 2)]))';
  end
  while true
    if strcmp(opt.label_names{opt.id(1)}, 'wall') && opt.plane(2)==1
      pos = my_impoly(old_p, snap, false);
    elseif strcmp(opt.label_names{opt.id(1)}, 'ceiling') && abs(opt.plane(3))==1
      pos = my_impoly(old_p, snap, false);
    else
      pos = my_impoly(old_p, snap);
    end
    n = size(pos, 1);
    if n>=2
      break;
    else
      clc; fprintf('Less than 2 points.\n');
    end
  end
  current_snap = {};
  for i=1:n
    current_snap{i} = [pos(i,:);pos(mod(i, n)+1, :)];
  end
  pos3d = (pos-repmat(opt.trans.T, [1 size(pos, 1)])')/opt.trans.S*[opt.trans.proj_x; opt.trans.proj_y];
  pos3d = pos3d + repmat(-opt.plane(4)*opt.plane(1:3), [size(pos, 1), 1]);
  for i=1:size(pos3d,1)
    polygon.pts{i}.x=pos3d(i, 1);
    polygon.pts{i}.y=pos3d(i, 2);
    polygon.pts{i}.z=pos3d(i, 3);
  end
end

function snap = helper_snapping(model, opt)
  trans = opt.trans;
  % project all layout elements for snapping
  if isfield(model, 'objects')
    n_snap = 0;
    snap = {};
    for i=1:numel(model.objects)
      if opt.id(1)==model.objects{i}.uid, continue; end
      
        for j=1:numel(model.objects{i}.model.surfaces)
          s = model.objects{i}.model.surfaces{j};
          s_pts = [s.polygon.pts{:}];
          pts = [s_pts.x; s_pts.y; s_pts.z];
          pts = ([trans.proj_x*pts; trans.proj_y*pts]*trans.S+repmat(trans.T, [1 size(s_pts, 2)]))';
          for k=1:size(pts, 1)
            n_snap = n_snap+1;
            snap{n_snap} = [pts(k, :); pts(mod(k, size(pts, 1))+1, :)];
          end
        end
      
    end
  else
    snap = {};
    return;
  end
end

function [model, opt] = annotate_floor(model, opt)
  has_floor = ismember('floor', opt.label_names);
  X = model.data.coord(:,:,1);
  Y = model.data.coord(:,:,2);
  Z = model.data.coord(:,:,3);
  
  if has_floor
    fprintf('Visible floor segment found\n');
    [~, floor_seg] = ismember(opt.label_names, {'floor'});
    floor_seg = find(floor_seg);
    [~, floor_mask]=ismember(opt.gt, floor_seg);
    floor_mask = logical(floor_mask);
    floor_plane = [ 0 1 0 -median(Y(floor_mask))];
    
    fprintf('Sainity check: camera height = %.3f m\n', floor_plane(4) );
    fprintf('      "enter" to accept current estimation\n');
    img_reshape = reshape(model.data.image, [opt.W*opt.H 3]);
    floor_colors = double(median(img_reshape(floor_mask, :), 1));
  else
    fprintf('Visible floor segment not found. Press \n');
  end
  
  fprintf('      "e" and then click on a point and specify the height.\n');
  while 1
    [x, y, bt] = ginput(1);
    if isempty(bt) && has_floor
      break;
    elseif bt=='e'
      fprintf('Now choose a point.\n');
      figure(opt.fig_frontal);
      try
      [x, y] = ginput(1);
      x = round(x); y = round(y);
      floor_plane = [ 0 1 0 -Y(y,x)];
      s = input('Enter the height of this plane: ', 's');
      A = sscanf(s, '%f');
      if (isempty(A))
        clc; fprintf('Bad input.\n');
        continue;
      end
      catch exp1
        clc; fprintf('Bad input.\n');
        continue;
      end
      floor_plane(4) = floor_plane(4) + A;
      fprintf('Camera height = %.3f m\n', floor_plane(4) );
      floor_colors = [.8 .8 .8];
      break;
    end
  end
  o = struct;
  o.uid = 0;
  o.type = 'ExtrudedObject';
  floor_colors = floor_colors;
  o.model.app.color.x = floor_colors(1);
  o.model.app.color.y = floor_colors(2);
  o.model.app.color.z = floor_colors(3);
  [xmin, xmax] = pminmax(X(:), 0);
  [zmin, zmax] = pminmax(Z(:), 0);
  t = [xmin xmin xmax xmax; ...
    -floor_plane(4) -floor_plane(4) -floor_plane(4) -floor_plane(4); ...
    zmin zmax zmax zmin];
  for i=1:4
    o.model.surfaces{1}.polygon.pts{i}.x = t(1,i);
    o.model.surfaces{1}.polygon.pts{i}.y = t(2,i);
    o.model.surfaces{1}.polygon.pts{i}.z = t(3,i);
  end
  o.model.surfaces{1}.plane = floor_plane;
  o.model.type = 'layout';
  o.model.label = 'floor';
  model.objects{1} = o;
end

function opt = disp_orth(model, opt)
  [proj_y, proj_x] = helper_surfaceproj(opt.plane);
  
  p3d = reshape(model.data.coord, [opt.W*opt.H 3])';
  proj = [proj_x*p3d; proj_y*p3d];
  imgproj = model.data.image;
  for ch=1:3
    I = model.data.image(:,:,ch);
    if ch==2
      I = I.^.8 + opt.mask;
    else
      I = I.^.8 - opt.mask;
    end
    imgproj(:,:,ch) = I;
  end
  
  [xmin, xmax] = pminmax(proj(1,:));
  [ymin, ymax] = pminmax(proj(2,:));
  s = (opt.work_size-1)/max(xmax-xmin, ymax-ymin);
  trans.T = [-xmin*s+opt.work_border; -ymin*s+opt.work_border];
  trans.S = s;
  trans.proj_y = proj_y;
  trans.proj_x = proj_x;
  
  opt.trans = trans;
  proj=round(proj*s+repmat(trans.T, [1 size(proj,2)]));
  proj(proj<1)=1;
  proj(proj>(opt.work_border*2+opt.work_size))=opt.work_border*2+opt.work_size;

  Y = model.data.coord(:,:,2); Y = Y(:) - 100*opt.mask(:);
  [~, order] = sort(Y);
  imgproj = reshape(imgproj, [opt.W*opt.H 3]);
  work_img = zeros((opt.work_size+2*opt.work_border)*(opt.work_size+2*opt.work_border), 3);
  work_img(proj(2, order) + (proj(1, order)-1)*(opt.work_border*2+opt.work_size),:)=imgproj(order, :);
  work_img = reshape(work_img, [opt.work_size+2*opt.work_border opt.work_size+2*opt.work_border, 3]);
  imshow(work_img);
  set(gca, 'Position', [0 0 1 1]);
  
  opt.trans = trans;
end

function opt = disp_frontal(model, opt)
  % display the frontal image for segment selection
  instanceEdge = zeros(opt.H, opt.W);
  if ~isfield(opt, 'gt')
    for i=1:numel(model.data.info)
      if strcmp(model.data.info{i}.key, 'gt')
        opt.gt = int32(model.data.info{i}.value);
      elseif strcmp(model.data.info{i}.key, 'label_names')
        r = model.data.info{i}.value;
        j=1;
        while ~isempty(r)
          [opt.label_names{j}, r] = strtok(r, ',');
          j=j+1;
        end
      elseif strcmp(model.data.info{i}.key, 'rawDepths')
        opt.rawDepths = model.data.info{i}.value;
      end
    end
    
    for j=1:numel(opt.label_names)
      eim = opt.gt==j;
      eim = (imdilate(eim, strel('disk', 2))-imerode(eim, strel('disk',2)))>0;
      instanceEdge(eim) = j;
    end
    opt.frontal_im = model.data.image;

    coloring = [0 0 0; hsv(numel(opt.label_names))];
    for ch=1:3
      t = coloring(:,ch);
      opt.frontal_im(:,:,ch) = (model.data.image(:,:,ch)*.7+.1).*(0.5+0.5*(opt.rawDepths>0)) + .3*t(instanceEdge+1);
    end
  end
  figure(opt.fig_frontal);
    
  im = opt.frontal_im;
  for ch=1:3
    I = opt.frontal_im(:,:,ch);
    if ch==2
      I(opt.mask) = I(opt.mask)*.5 + .5;
    end
    im(:,:,ch) = I;
  end
  imshow(im);
  set(gca, 'Position', [0 0 1 1]);
end

function [fmin, fmax] = pminmax(arr, tolerance)
  if nargin==1
    tolerance=0.005;
  end
  sorted_arr = sort( arr );
  fmin = sorted_arr(max(1,ceil(tolerance*numel(arr))));
  fmax = sorted_arr(min(numel(arr), floor((1-tolerance)*numel(arr))));
end