function scenemodel_render(model, opt)
  warning off;
  if ~isstruct(model)
    model = scenemodel_import(model);
  end

  if nargin<2
    opt = get_opt();
  end
  
  set(gcf, 'Render', 'zbuffer');
  set(gcf, 'KeyPressFcn', @myCallback);
  set(gcf, 'Color', [0 0 0]);
  %set(gcf, 'WindowStyle', 'Modal');
  hold on;
  
  % lights
  l = inf;
  u0 = model.camera.K(1,3);
  v0 = model.camera.K(2,3);
  fx = model.camera.K(1,1);
  fy = model.camera.K(2,2);
  s = opt.default_scale;
  
  if opt.render_pointcloud
    ssize = size(model.data.coord);
    model.data.coord = reshape(model.data.coord, [ssize(1)*ssize(2) 3]);
    model.data.coord = model.camera.R*model.data.coord';
    model.data.coord = reshape(model.data.coord', ssize);
    model.data.coord = model.data.coord * opt.render_scale;
    h = surf(model.data.coord(:,:,1), model.data.coord(:,:,2), model.data.coord(:,:,3), model.data.image, ...
        'FaceColor', 'texturemap', 'EdgeColor','none', 'CDataMapping','direct');
  else
    for i_obj=1:numel(model.objects)
      o = model.objects{i_obj};
      if ~isfield(o, 'mesh'), continue; end
      color = [o.mesh.app.color.x o.mesh.app.color.y o.mesh.app.color.z];
      for i_comp=1:numel(o.mesh.comp)
        T=patch('vertices', o.mesh.comp{i_comp}.vertices*model.camera.R'*opt.render_scale, 'faces', o.mesh.comp{i_comp}.faces, ...
        'facecolor', [o.mesh.app.color.x o.mesh.app.color.y o.mesh.app.color.z]);
      end
    end
  end
  
  %shading interp;
%   if isinf(l)
%     l = camlight('left');
%   else
%     delete(l); l = inf;
%   end
  set(gca, 'DataAspectRatio', [1 1 1]);
  axis vis3d off;
  set(gca, 'CameraPositionMode', 'manual');
  set(gca, 'CameraTargetMode', 'manual');
  set(gca, 'CameraViewAngleMode', 'manual');
  set(gca, 'CameraUpVectorMode', 'manual');
  
  set(gca, 'CameraPosition', [model.camera.Origin.x model.camera.Origin.y model.camera.Origin.z]);
  set(gca, 'CameraTarget', [0 0 -1]);
  set(gca, 'CameraUpVector', [0 1 0]);
  % Ref: http://en.wikipedia.org/wiki/Kinect
  set(gca, 'Projection', 'perspective');
  % do the math to find out view angle
  set(gca, 'CameraViewAngle', atan2(1, fy*opt.W/opt.H)*180/pi*2 );
  set(gcf, 'Position', [0 0 s s*opt.H/opt.W]);
  %set(gca, 'Position', [.01 0.01 1 1]);
  set(gca, 'Position', [u0/2 -v0/2 1 1]);
  % animate it
  lock = 0;
  
  
  function myCallback(src,eventdata)
    % current target-pos
    if lock~=0
      return;
    end
    lock = 1;
    cz = camtarget - campos;
    cy = camup;
    cx = cross(cz, cy);
    dist_step = .1*100;
    % increase 5 degrees, do the math
    angle_step = sin(5/180*pi)/sin(130/180*pi)/sqrt(2);
    if strcmp(eventdata.Key, 'downarrow')
      campos(-dist_step*cz+campos);
      camtarget(campos+cz);
    elseif strcmp(eventdata.Key, 'uparrow')
      campos(dist_step*cz+campos);
      camtarget(campos+cz);
    elseif strcmp(eventdata.Key, 'rightarrow')
      cz_p = cz + (cx-cz)*angle_step; cz_p = cz_p / sqrt(sum(cz_p.^2));
      camtarget(campos + cz_p);
      camup(cy);
    elseif strcmp(eventdata.Key, 'leftarrow')
      cz_p = cz + ((-cx)-cz)*angle_step; cz_p = cz_p / sqrt(sum(cz_p.^2));
      camtarget(campos + cz_p);
      camup(cy);
    elseif strcmp(eventdata.Key, 's')
      cz_p = cz + ((-cy)-cz)*angle_step; cz_p = cz_p / sqrt(sum(cz_p.^2));
      cy_p = cy + ((cz)-cy)*angle_step; cy_p = cy_p / sqrt(sum(cy_p.^2));
      camtarget(campos + cz_p);
      camup(cy_p);
    elseif strcmp(eventdata.Key, 'w')
      cz_p = cz + (cy-cz)*angle_step; cz_p = cz_p / sqrt(sum(cz_p.^2));
      cy_p = cy + ((-cz)-cy)*angle_step; cy_p = cy_p / sqrt(sum(cy_p.^2));
      camtarget(campos + cz_p);
      camup(cy_p);
    elseif strcmp(eventdata.Key, 'a')
      campos(-dist_step*cx+campos);
      camtarget(campos+cz);
    elseif strcmp(eventdata.Key, 'd')
      campos(dist_step*cx+campos);
      camtarget(campos+cz);
    elseif strcmp(eventdata.Key, 't')
      if isinf(l)
        l = camlight('left');
      else
        delete(l); l = inf;
      end
    else
      return;
    end
    lock = 0;
  end
  
end