function cmap = scenemodel_renderlayout(model, opt)
  warning off;
  if ~isstruct(model)
    model = scenemodel_import(model);
  end

  if nargin<2
    opt = get_opt();
  end
  clf;
  set(gcf, 'Render', 'zbuffer');
  set(gcf, 'Color', [0 0 0]);
  %set(gcf, 'WindowStyle', 'Modal');
  hold on;
  
  % lights
  l = inf;
  u0 = model.camera.K(1,3);
  v0 = model.camera.K(2,3);
  fx = model.camera.K(1,1);
  fy = model.camera.K(2,2);
  s = opt.default_scale;
  
  if opt.render_pointcloud
    ssize = size(model.data.coord);
    model.data.coord = reshape(model.data.coord, [ssize(1)*ssize(2) 3]);
    model.data.coord = model.camera.R*model.data.coord';
    model.data.coord = reshape(model.data.coord', ssize);
    model.data.coord = model.data.coord * opt.render_scale;
    h = surf(model.data.coord(:,:,1), model.data.coord(:,:,2), model.data.coord(:,:,3), model.data.image, ...
        'FaceColor', 'texturemap', 'EdgeColor','none', 'CDataMapping','direct');
  else
    for i_obj=1:numel(model.objects)
      o = model.objects{i_obj};
      if ~strcmp(model.objects{i_obj}.model.type, 'layout'), continue; end
      if ~isfield(o, 'mesh'), continue; end
      
      if strcmp(o.model.label, 'wall')
        color = [1 0 0];
      elseif strcmp(o.model.label, 'ceiling')
        color = [0 1 0];
      elseif strcmp(o.model.label, 'floor')
        color = [0 0 1];
      end
      %color = [o.mesh.app.color.x o.mesh.app.color.y o.mesh.app.color.z];
      for i_comp=1:numel(o.mesh.comp)
        T=patch('vertices', o.mesh.comp{i_comp}.vertices*model.camera.R'*opt.render_scale, 'faces', o.mesh.comp{i_comp}.faces, ...
        'facecolor', color, 'EdgeColor','none');
      end
    end
  end
  
  %shading interp;
%   if isinf(l)
%     l = camlight('left');
%   else
%     delete(l); l = inf;
%   end
  set(gca, 'DataAspectRatio', [1 1 1]);
  axis vis3d off;
  set(gca, 'CameraPositionMode', 'manual');
  set(gca, 'CameraTargetMode', 'manual');
  set(gca, 'CameraViewAngleMode', 'manual');
  set(gca, 'CameraUpVectorMode', 'manual');
  
  set(gca, 'CameraPosition', [model.camera.Origin.x model.camera.Origin.y model.camera.Origin.z]);
  set(gca, 'CameraTarget', [0 0 -1]);
  set(gca, 'CameraUpVector', [0 1 0]);
  % Ref: http://en.wikipedia.org/wiki/Kinect
  set(gca, 'Projection', 'perspective');
  % do the math to find out view angle
  set(gca, 'CameraViewAngle', atan2(1, fy*opt.W/opt.H)*180/pi*2 );
  %set(gcf, 'Position', [0 0 s s*opt.H/opt.W]);
  %set(gca, 'Position', [.01 0.01 1 1]);
  set(gca, 'Position', [u0/2 -v0/2 1 1]);
  % animate it
  f = getframe;
  cmap = f.cdata;
end