function model = scenemodel_translate(model, opt)
  % translate extruded/sketchup models to meshes
  if ~isfield(model, 'objects')
    return;
  end
  for i=1:numel(model.objects)
    o = model.objects{i};
    % from extruded to surfaces
    if strcmp(o.type, 'ExtrudedObject') && strcmp(o.model.type, 'object')
      m = o.model;
      s_pts = [m.horizontal2D.pts{:}];
      h_pts = [s_pts.x; s_pts.y; s_pts.z];
      extrusion = m.projection2D(1:3)*h_pts;
      s_pts = [m.overhead2D.pts{:}];
      o_pts = [s_pts.x; s_pts.y; s_pts.z];
      top_pts = repmat(m.projection2D(1:3)*min(extrusion), [size(o_pts, 2) 1])' + o_pts;
      bottom_pts = repmat(m.projection2D(1:3)*max(extrusion), [size(o_pts, 2) 1])' + o_pts;
      s = cell(1);
      n = numel(m.overhead2D.pts);
      for j=1:n
        prev = j; next = mod(j,n)+1;
        t = [top_pts(:,prev) top_pts(:,next) bottom_pts(:,next) bottom_pts(:,prev)];
        for k=1:4
          s{j}.polygon.pts{k}.x = t(1,k);
          s{j}.polygon.pts{k}.y = t(2,k);
          s{j}.polygon.pts{k}.z = t(3,k);
        end
        [u d v]=svd([t' ones(size(t,2),1)]);
        s{j}.plane = v(:,4);
        s{j}.plane = s{j}.plane / sqrt( sum(s{j}.plane(1:3).^2) );
      end
      for k=1:n
        s{n+1}.polygon.pts{k}.x = top_pts(1,k); s{n+1}.polygon.pts{k}.y = top_pts(2,k); s{n+1}.polygon.pts{k}.z = top_pts(3,k);
        s{n+2}.polygon.pts{k}.x = bottom_pts(1,k); s{n+2}.polygon.pts{k}.y = bottom_pts(2,k); s{n+2}.polygon.pts{k}.z = bottom_pts(3,k);
      end
      
      [u d v]=svd([top_pts' ones(size(top_pts,2),1)]);
      s{n+1}.plane = v(:,4);
      s{n+1}.plane = s{n+1}.plane / sqrt( sum(s{n+1}.plane(1:3).^2) );
      [u d v]=svd([bottom_pts' ones(size(bottom_pts,2),1)]);
      s{n+2}.plane = v(:,4);
      s{n+2}.plane = s{n+2}.plane / sqrt( sum(s{n+2}.plane(1:3).^2) );
      
      o.model.surfaces = s;
      
    end
    
    % from surfaces to meshes
    
    if strcmp(o.model.type, 'furniture')
      A = load(['model/' o.model.basemodel]);
      R = o.model.R;
      
      o_pts = [o.model.overhead2D.pts{:}];
      h_pts = [o.model.horizontal2D.pts{:}];
      v = [o_pts.x; o_pts.y; o_pts.z]';
      new_min([1,3]) = min(v(:,[1,3])); new_max([1,3]) = max(v(:,[1,3])); 
      new_t = mean([new_min; new_max]);
      v = (v - repmat(new_t, [size(v, 1) 1]))*R' + repmat(new_t, [size(v, 1) 1]);
      new_min([1,3]) = min(v(:,[1,3])); new_max([1,3]) = max(v(:,[1,3])); 
      
      v = [h_pts.x; h_pts.y; h_pts.z]';
      new_min(2) = min(v(:,2)); new_max(2) = max(v(:,2)); 
      new_t = mean([new_min; new_max]);
      o.mesh = {};
      for j=1:numel(A.comp)
        o.mesh.comp{j}.vertices = (A.comp{j}.vertices) ...
          .* repmat(new_max-new_min, [size(A.comp{j}.vertices, 1) 1])*R ...
          + repmat(new_t, [size(A.comp{j}.vertices, 1) 1]);
        o.mesh.comp{j}.faces = A.comp{j}.faces;
      end
      o.mesh.app = o.model.app;
      o.model.surfaces = {};
      
      for j=1:numel(A.surfaces)
        s = A.surfaces{j} ...
          .* repmat(new_max-new_min, [size(A.surfaces{j}, 1) 1])*R ...
          + repmat(new_t, [size(A.surfaces{j}, 1) 1]);
        for u=1:size(s,1)
          o.model.surfaces{j}.polygon.pts{u}.x = s(u,1);
          o.model.surfaces{j}.polygon.pts{u}.y = s(u,2);
          o.model.surfaces{j}.polygon.pts{u}.z = s(u,3);
        end
        [u d v]=svd([s ones(size(s',2),1)]);
        o.model.surfaces{j}.plane = v(:,4);
        o.model.surfaces{j}.plane = o.model.surfaces{j}.plane / sqrt( sum(o.model.surfaces{j}.plane(1:3).^2) );
      end
    elseif strcmp(o.model.type, 'layout') || strcmp(o.model.type, 'object')
      m = struct;
      for j=1:numel( o.model.surfaces )
        s = o.model.surfaces{j};
        [proj_y, proj_x] = helper_surfaceproj( s.plane );
        % do Delaunay triangulation to display surface with holes
        s_pts = [s.polygon.pts{:}];
        v = [s_pts.x; s_pts.y; s_pts.z]';
        n_opening = 0;
        outv_2d = [v*proj_x' v*proj_y'];
        if isfield(s, 'opening')
          n_opening = numel(s.opening);
          for k=1:n_opening
            s_pts = [s.opening{k}.pts{:}];
            vt = [s_pts.x; s_pts.y; s_pts.z]';
            v = [v; vt];
          end
        end
        v_2d = [v*proj_x' v*proj_y'];
        
        r = DelaunayTri(v_2d);
        n = size(r.Triangulation, 1);
        is_hole = true(n, 1);
        center_pos = zeros(n, 2);
        for k=1:n
          center_pos(k, :) = mean(r.X(r.Triangulation(k,:), :));
        end
        is_hole = inpolygon(center_pos(:,1), center_pos(:,2), outv_2d(:,1), outv_2d(:,2) );
        for k=1:n_opening
          s_pts = [s.opening{k}.pts{:}];
          vt = [s_pts.x; s_pts.y; s_pts.z]';
          vt = [vt*proj_x' vt*proj_y'];
          is_hole = is_hole & ~inpolygon(center_pos(:,1), center_pos(:,2), vt(:,1), vt(:,2) );
        end
        
        [~, idx] = min(pdist2(v_2d, r.X));
        m.comp{j}.vertices = v(idx, :);
        m.comp{j}.faces = r.Triangulation(is_hole, :);
      end
      m.app = o.model.app;
      m.uid=i;
      o.mesh = m;
      
    end
    model.objects{i} = o;
  end
end