addpath('process');
if ~exist('scenes')
  load data/complete/nyu_depth_v2_labeled;
  load data/complete/support_labels;
end

scene_names = unique(scenes);
save('data/basic.mat', 'names', 'namesToIds', 'scene_names');
fprintf('The size of scenes = %d\n', numel(scene_names));
Nscenes = size(scene_names);
Nimages = size(scenes);

h = java.util.Hashtable;
for i=1:Nimages
  
  t = {};
  c = h.get(scenes{i});
  if isempty(c)
    c = 1;
  else
    c = c + 1;
  end
  fname = sprintf('%d_%s_%d.mat', i, scenes{i}, c);
  
  h.put(scenes{i}, c);
  t.accelData = accelData(i, :);
  t.depths = depths(:,:,i);
  t.instances = instances(:,:,i);
  t.labels = labels(:,:,i);
  t.rawDepthFilenames = rawDepthFilenames(i);
  t.rawRgbFilenames = rawRgbFilenames(i);
  t.scenes = scenes(i);
  t.images = images(:,:,:,i);
  t.rawDepths = rawDepths(:,:,i);
  t.sceneTypes = sceneTypes(i);
  
  t.images = crop_image(t.images);
  t.depths = crop_image(t.depths);
  t.rawDepths = crop_image(t.rawDepths);
  t.labels = crop_image(t.labels);
  t.instances = crop_image(t.instances);
  % process support
  t.support = supportLabels{i};
  MIN_PIXELS_PER_REGION = 10;
  imgRegions = get_regions_from_labels(t.labels, t.instances);
  [~, R] = get_region_ids(imgRegions);
  
  for rr = 1 : R
    regionMask = imgRegions == rr;
    
    if nnz(regionMask) < MIN_PIXELS_PER_REGION
      fprintf('Discarding a region with less than %d pixels.\n', MIN_PIXELS_PER_REGION);
      imgRegions(regionMask) = 0;
    end
  end
  
  [regionIds, R] = get_region_ids(imgRegions);
  
  for rr = 1 : R
    regionMask = imgRegions == regionIds(rr);
    imgRegions(regionMask) = rr;
  end
  
  [regionIds, R] = get_region_ids(imgRegions);
  assert(max(imgRegions(:)) == R);
  assert(max(imgRegions(:)) == numel(regionIds));

  t.gt = imgRegions;
  label_names = {};
  for rr = 1:R
    regionMask = imgRegions == rr;
    
    u = unique(t.labels( regionMask ));
    assert(numel(u)>0);
    label_names{rr} = names{u(end)};
  end
  t.label_names = label_names;
  data = t;
  data = processRoom(data);
  fprintf('Saving %s : %d / %d \n', fname, i, Nimages);
  save(['processed_data/' fname], 'data');
end