function [planeim, normalim, confim] = computeLocalPlanes(X, Y, Z)


bw = [1 3 6 9]; % blockwidth
depththresh = 0.05; % relative depth threshold

[imh, imw] = size(Z);
npix = imh*imw;

pts = [X(:) Y(:) Z(:) ones(npix, 1)];

[u, v] = meshgrid(1:imw, 1:imh);

[nu, nv] = meshgrid([-bw 0 bw], [-bw 0 bw]);

nx = zeros(imh, imw);
ny = zeros(imh, imw);
nz = zeros(imh, imw);
nd = zeros(imh, imw);
confim = zeros(imh, imw);
%planeim = zeros(imh, imw, 4);

ind_all = find(Z);
for k = ind_all(:)'        
    
    u2 = u(k)+nu;
    v2 = v(k)+nv;
    
    % check that u2 and v2 are in image
    valid = (u2 > 0) & (v2 > 0) & (u2 <= imw) & (v2 <= imh);
    u2 = u2(valid);
    v2 = v2(valid);
    ind2 = v2 + (u2-1)*imh;
    
    % check that depth difference is not too large
    %valid = min(abs(pts(ind2, 1:3)-pts(k, 1:3)), 2) < Z(k)*depththresh; %Z(ind2)-Z(k)) < Z(k)*depththresh;
    valid = abs(Z(ind2)-Z(k)) < -Z(k)*depththresh;
    %valid = abs(Z(ind2)-Z(k)) < (Z(k).^2*0.0075);
    u2 = u2(valid);
    v2 = v2(valid);
    ind2 = v2 + (u2-1)*imh;
    
    if numel(u2)<6, continue; end
    
    A = pts(ind2, :);        
%     [U S V] = svd(A);
%     nx(k)=V(1,end);
%     ny(k)=V(2,end);
%     nz(k)=V(3,end);
%     nd(k)=V(4,end);
%     confim(k) = 1-sqrt(S(4,4)/S(3,3));
    [eigv, l] = eig(A'*A);
    nx(k) = eigv(1,1);
    ny(k) = eigv(2,1);
    nz(k) = eigv(3,1);
    nd(k) = eigv(4,1);
    confim(k) = 1- sqrt(l(1) / l(2,2)); 
    
end

% normalize so that first three coordinates form a unit normal vector and
% the largest normal component is positive
planeim = cat(3, nx, ny, nz, nd); % ./ repmat(len, [1 1 4]);
len = sqrt(nx.^2+ny.^2+nz.^2);
planeim = planeim ./ repmat(len+eps, [1 1 4]);
normalim = planeim(:, :, 1:3);
normalim = -normalim.*repmat(sign(sum(cat(3, X, Y, Z).*normalim, 3)), [1 1 3]);
%normalim = planeim(:, :, 1:3) ./ repmat(len+eps, [1 1 3]);

% tmp = max(planeim(:, :, 1:3), [], 3) < max(-planeim(:, :, 1:3), [], 3);
% planeim(repmat(tmp, [1 1 4])) = -planeim(repmat(tmp, [1 1 4]));

